@@ -3,7 +3,7 @@
 
 #########################
 
-use Test::Simple tests => 8;
+use Test::Simple tests => 11;
 
 use Perl6::Contexts;
 
@@ -27,6 +27,11 @@ ok($foo->[1] == $bar[1], 'reference context - arrays');
 
 # 3
 
+$bar[6] = @bar;
+ok($bar[6]->[3] == $bar[3], 'reference context - arrays part 2');
+
+# 4
+
 $foo = %baz;
 die unless $foo;
 die unless ref $foo;
@@ -34,12 +39,13 @@ die unless keys %baz;
 die unless exists $baz{baz};
 ok($foo->{baz} eq $baz{baz}, 'reference context - hashes');
 
-# 4
+# 5 
 
+@bar = (1 .. 5);
 $foo = 0 + @bar;
 ok($foo == 5, 'numeric context - math ops');
 
-# 5
+# 6
 
 ok(scalar(@bar) == 5, 'numeric context - scalar keyword');
 
@@ -49,23 +55,27 @@ ok(scalar(@bar) == 5, 'numeric context - scalar keyword');
 # use autobox::Core;
 # ok(@foo->size, '5');
 
-# 6
+# 7
 
 $foo = bjork(10, 20, @bar);
 ok($foo eq "numargs 3\n", 'subroutine arguments');
 
-# 7
+# 8
 
 $foo = blurgh->bjork(30, 40, @bar);
 ok($foo eq "numargs 4\n", 'method arguments');
 
-# 8
+# 9
 
 local $" = ' ';
 # same as this, by the way: print 'foo' . join(${'"'}, @arr) . "\n";
 $foo = 'foo' . @bar . "\n";
 ok($foo eq "foo1 2 3 4 5\n", 'scalar context - arrays');
 
+# 10, 11
+
+ok(gnrash(\@bar, \@bar), 'multiple arrays as args control');
+ok(gnrash(@bar, @bar), 'multiple arrays as args test');
 
 
 #
@@ -76,6 +86,10 @@ sub bjork {
     return join '', "numargs ", scalar @_, "\n";
 }
 
+sub gnrash {
+    ref $_[0] eq 'ARRAY' and ref $_[1] eq 'ARRAY';
+}
+
 package blurgh;
 
 sub bjork {
